from typing import Protocol
from urllib import parse
import pathlib
from .extractor import Extractor, TextExtractor, YoutubeSubtitleExtractor


class Resource:
    """This class parses selected resource identifiers (e.g. URL's, file paths), and chooses the appropriate text extractor."""

    def __init__(self, identifier: str) -> None:
        self._identifier = identifier

    def get_extractor(self) -> Extractor:
        if (url := parse.urlparse(self._identifier)).netloc:
            return self._get_extractor_from_url(url)
        elif (path := pathlib.Path(self._identifier)).exists():
            return self._get_extractor_from_path(path)
        else:
            raise Exception(
                f"'{self._identifier}' is not a valid URL or a path to an existing file."
            )

    def _get_extractor_from_url(self, url: parse.ParseResult) -> Extractor:
        match url.hostname:
            case "www.youtube.com":
                return self._get_youtube_extractor(url)
            case _:
                return TextExtractor("Found url")

    def _get_youtube_extractor(self, url: parse.ParseResult) -> Extractor:
        try:
            return YoutubeSubtitleExtractor(video_id=parse.parse_qs(url.query)["v"][0])
        except KeyError:
            raise Exception(f"'{url.geturl()}' is missing the 'v' query parameter.")

    def _get_extractor_from_path(self, path: pathlib.Path) -> Extractor:
        return TextExtractor("Found path")



from typing import Protocol, Iterator, TextIO
import click

from .resource import Resource
from .extractor import TextExtractor


class CLI:
    """This class has the core functions that should be exposed to a command-line user.

    To make a CLI application, this class must be wrapped by a command-line adapter.
    """

    def __init__(self) -> None:
        pass

    def extract_text_from_resource(self, resource: str) -> Iterator[str]:
        """Attempts to extract the spoken or written content of the resource and return it as plain text. Examples:
        * `extract_text_from_resource("https://www.youtube.com/watch?v=NiTsduRreug")`:  attempts to extract a transcript of the video.
        * `extract_text_from_resource("https://en.wikipedia.org/wiki/Stephen_Krashen")`:  attempts to extract the article text of the web page.
        * `extract_text_from_resource("my_book.epub:2")`: attempts to extract the second chapter of the given `epub` and convert it to plain text.

        :param resource: A URL or a file path pointing to the resource.
        :return: The extracted plain text of the resource.
        """
        ...
        extractor = Resource(resource).get_extractor()
        text = extractor.extract()
        for line in text:
            yield line

    def extract_text_from_stream(self, stream: TextIO) -> Iterator[str]:
        """Converts an input stream to plain text.

        :param stream: A text stream.
        :return: The input stream converted to plain text.
        """
        ...
        extractor = TextExtractor(stream)
        text = extractor.extract()
        for line in text:
            yield line


class CLIAdapter:
    """Command-line adapter that uses the click library."""

    def __init__(self, cli: CLI) -> None:
        self.cli: CLI = cli

    def _create_extract_command(self) -> click.Command:
        @click.command()
        @click.argument("resource", required=False)
        def extract(resource: str):
            if resource:
                extracted_text = self.cli.extract_text_from_resource(resource)
            else:
                stdin_text = click.get_text_stream("stdin")
                extracted_text = self.cli.extract_text_from_stream(stdin_text)

            for line in extracted_text:
                click.echo(line)

        return extract

    def run(self) -> None:
        cli = click.Group()
        cli.add_command(self._create_extract_command())
        cli()



from typing import Protocol, Iterator, Iterable
from youtube_transcript_api import YouTubeTranscriptApi

from .env import LANGUAGE


class Extractor(Protocol):
    """This class extracts plain text from a specific resource type containing spoken or written content."""

    def extract(self) -> Iterator[str]: ...


class TextExtractor:
    def __init__(self, text: Iterable[str]) -> None:
        self._text: Iterable[str] = text

    def extract(self) -> Iterator[str]:
        for line in self._text:
            yield line


class YoutubeSubtitleExtractor:
    def __init__(self, video_id: str) -> None:
        self._video_id: str = video_id

    def extract(self) -> Iterator[str]:
        fetched_transcript = YouTubeTranscriptApi().fetch(
            self._video_id, languages=[LANGUAGE]
        )
        for snippet in fetched_transcript:
            yield snippet.text
